<?php

/**
*
* @package BB3Ajaxer
* @copyright (c) 2023 PPK
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*
*/

namespace ppk\bb3ajaxer\event;

/**
 * @ignore
 */
if (!defined('IN_PHPBB'))
{
	exit;
}

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class listener implements EventSubscriberInterface
{

	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\request\request_interface */
	protected $request;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var string phpbb_root_path */
	protected $phpbb_root_path;

	/** @var string phpEx */
	protected $php_ext;

	/** @var \phpbb\extension\manager */
	protected $phpbb_extension_manager;

	public function __construct(\phpbb\user $user, \phpbb\auth\auth $auth, \phpbb\db\driver\driver_interface $db, \phpbb\template\template $template, \phpbb\request\request_interface $request, \phpbb\config\config $config, $phpbb_root_path, $php_ext, \phpbb\extension\manager $phpbb_extension_manager)
	{
		$this->user = $user;
		$this->auth = $auth;
		$this->db = $db;
		$this->template = $template;
		$this->request = $request;
		$this->config = $config;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->php_ext = $php_ext;
		$this->phpbb_extension_manager = $phpbb_extension_manager;

		$this->bb3ajaxer_config=json_decode($this->config['bb3ajaxer_config'], true);
		$this->bb3ajaxer_config_apply=json_decode($this->config['bb3ajaxer_config_apply'], true);
	}

	static public function getSubscribedEvents()
	{
		return array(
			'core.viewforum_modify_topics_data' => 'bb3ajaxer_viewforum',
			'core.viewtopic_assign_template_vars_before' => 'bb3ajaxer_viewtopic',
			'core.memberlist_modify_sort_pagination_params' => 'bb3ajaxer_memberlist',
			'core.search_backend_search_after' => 'bb3ajaxer_search',

			'core.ucp_prefs_view_data' => 'bb3ajaxer_ucp_data',
			'core.ucp_prefs_view_update_data' => 'bb3ajaxer_ucp_sql',
			'core.ucp_prefs_modify_common' => 'bb3ajaxer_ucp_tpl',

			'core.acp_board_config_edit_add'           => 'add_config',

			'core.acp_users_prefs_modify_data' => 'bb3ajaxer_ucp_data',
			'core.acp_users_prefs_modify_sql' => 'bb3ajaxer_ucp_sql',
			'core.acp_users_prefs_modify_template_data' => 'bb3ajaxer_ucp_tpl',

		);
	}

	public function bb3ajaxer_memberlist($event)
	{
		$total_users=$event['total_users'];

		if($total_users)
		{
			$user_option=$this->get_bb3ajaxer_config();

			$this->user->add_lang_ext('ppk/bb3ajaxer', 'bb3ajaxer');

			$start=$this->request->variable('start', 0);

			$per_page=$this->config['topics_per_page'];
			$num_items=$total_users;
			$current=$this->get_on_page($per_page, $start);
			$max=$this->on_page($num_items, $per_page, $start);

			$this->template->assign_vars(array(
				'S_BB3AJAXER_CURRENT' => $current,
				'S_BB3AJAXER_MAX' => $max,
				'S_BB3AJAXER_PER_PAGE' => $per_page,
				'S_BB3AJAXER_START' => $start,

				'U_BB3AJAXER_MEMBERS_PREV' => !$start || $user_option['prev']!=1 || $user_option['m']==-1 ? false : generate_board_url().'/memberlist.'.$this->php_ext."?start=",
				)
			);

			if($user_option['m']==-1)
			{
				return false;
			}
			else if($user_option['m']==1)
			{
				$this->template->assign_vars(array(
					'S_USER_BB3AJAXER_MEMBERS' => true,
					)
				);
			}
			else
			{
				$this->template->assign_vars(array(
					'U_BB3AJAXER_MEMBERS' => $current==$max ? false : generate_board_url().'/memberlist.'.$this->php_ext."?start=",
					)
				);
			}
		}
	}

	public function bb3ajaxer_search($event)
	{
		$total_match_count=$event['total_match_count'];

		if($total_match_count)
		{
			$user_option=$this->get_bb3ajaxer_config();

			$this->user->add_lang_ext('ppk/bb3ajaxer', 'bb3ajaxer');

			$start=$event['start'];

			$per_page=$this->config['topics_per_page'];
			$num_items=$total_match_count;
			$current=$this->get_on_page($per_page, $start);
			$max=$this->on_page($num_items, $per_page, $start);
			$start=$this->validate_start($start, $per_page, $num_items);

			$is_bb2spoiler_installed=($this->phpbb_extension_manager->is_enabled('ppk/bb2spoiler_se') || $this->phpbb_extension_manager->is_enabled('ppk/bb2spoiler')) ? true : false;
			$is_bb3spoiler_installed=$this->phpbb_extension_manager->is_enabled('ppk/bb3spoiler') ? true : false;
			$is_topicpreview_installed=$this->phpbb_extension_manager->is_enabled('vse/topicpreview') && $this->config['topic_preview_limit'] ? true : false;
			$is_whoposted_installed=$this->phpbb_extension_manager->is_enabled('rmcgirr83/whoposted') ? true : false;

			$this->template->assign_vars(array(
				'S_BB2SPOILER_INSTALLED' => $is_bb2spoiler_installed,
				'S_BB3SPOILER_INSTALLED' => $is_bb3spoiler_installed,
				'S_TOPICPREVIEW_INSTALLED' => $is_topicpreview_installed,
				'S_WHOPOSTED_INSTALLED' => $is_whoposted_installed,

				'S_FIRST_POST_ON_EVERY_PAGE' => 0,

				'S_BB3AJAXER_CURRENT' => $current,
				'S_BB3AJAXER_MAX' => $max,
				'S_BB3AJAXER_PER_PAGE' => $per_page,
				'S_BB3AJAXER_START' => $start,
				'S_BB3AJAXER_SORT' => $event['sort_dir'],

				'U_BB3AJAXER_SEARCH_PREV' => !$start || $user_option['prev']!=1 || $user_option['s']==-1 ? false : generate_board_url().'/search.'.$this->php_ext."?start=",
				)
			);

			if($user_option['s']==-1)
			{
				return false;
			}
			else if($user_option['s']==1)
			{
				$this->template->assign_vars(array(
					'S_USER_BB3AJAXER_SEARCH' => true,
					)
				);
			}
			else
			{
				$this->template->assign_vars(array(
					'U_BB3AJAXER_SEARCH' => $current==$max ? false : generate_board_url().'/search.'.$this->php_ext."?start=",
					)
				);
			}
		}
	}

	public function bb3ajaxer_viewforum($event)
	{
		$total_topic_count=$event['total_topic_count'];

		if($total_topic_count)
		{
			$user_option=$this->get_bb3ajaxer_config();

			$this->user->add_lang_ext('ppk/bb3ajaxer', 'bb3ajaxer');

			$forum_id=$event['forum_id'];
			$start=$this->request->variable('start', 0);

			$per_page=$this->config['topics_per_page'];
			$num_items=$total_topic_count;
			$current=$this->get_on_page($per_page, $start);
			$max=$this->on_page($num_items, $per_page, $start);

			$is_topicpreview_installed=$this->phpbb_extension_manager->is_enabled('vse/topicpreview') && $this->config['topic_preview_limit'] ? true : false;
			$is_whoposted_installed=$this->phpbb_extension_manager->is_enabled('rmcgirr83/whoposted') ? true : false;

			$this->template->assign_vars(array(
				'S_TOPICPREVIEW_INSTALLED' => $is_topicpreview_installed,
				'S_WHOPOSTED_INSTALLED' => $is_whoposted_installed,

				'S_BB3AJAXER_CURRENT' => $current,
				'S_BB3AJAXER_MAX' => $max,
				'S_BB3AJAXER_PER_PAGE' => $per_page,
				'S_BB3AJAXER_START' => $start,

				'U_BB3AJAXER_TOPICS_PREV' => !$start || $user_option['prev']!=1 || $user_option['t']==-1 ? false : generate_board_url().'/viewforum.'.$this->php_ext."?f=$forum_id&start=",
				)
			);

			if($user_option['t']==-1)
			{
				return false;
			}
			else if($user_option['t']==1)
			{
				$this->template->assign_vars(array(
					'S_USER_BB3AJAXER_TOPICS' => true,
					)
				);
			}
			else
			{
				$this->template->assign_vars(array(
					'U_BB3AJAXER_TOPICS' => $current==$max ? false : generate_board_url().'/viewforum.'.$this->php_ext."?f=$forum_id&start=",
					)
				);
			}
		}
	}

	public function bb3ajaxer_viewtopic($event)
	{
		$total_posts=$event['total_posts'];
		$topic_data=$event['topic_data'];
		$base_url=$event['base_url'];

		$is_quickreply_installed=$this->phpbb_extension_manager->is_enabled('boardtools/quickreply') ? true : false;
		$qr_ajax_submit=$is_quickreply_installed && $this->config['qr_ajax_submit'] ? true : false;
		$qr_ajax_pagination=$is_quickreply_installed && $this->config['qr_ajax_pagination'] && $this->user->data['ajax_pagination'] ? true : false;

		$is_firstmessageonly_installed=!$this->user->data['is_registered'] && $this->phpbb_extension_manager->is_enabled('ppk/firstmessageonly') ? true : false;
		$is_firstpageonly_installed=!$this->user->data['is_registered'] && $this->phpbb_extension_manager->is_enabled('ppk/firstpageonly') ? true : false;

		if($total_posts && !$qr_ajax_submit && !$qr_ajax_pagination && !$is_firstmessageonly_installed && !$is_firstpageonly_installed)
		{
			$user_option=$this->get_bb3ajaxer_config();

			$this->user->add_lang_ext('ppk/bb3ajaxer', 'bb3ajaxer');

			$forum_id=$event['forum_id'];
			$topic_id=$event['topic_id'];
			$start=$event['start'];

			$per_page=$this->config['posts_per_page'];
			$num_items=$total_posts;
			$current=$this->get_on_page($per_page, $start);
			$max=$this->on_page($num_items, $per_page, $start);
			$start=$this->validate_start($start, $per_page, $num_items);
			$sort_dir=strpos($base_url, 'sd=d')!==false ? 'd' : 'a';

			$is_bb2spoiler_installed=($this->phpbb_extension_manager->is_enabled('ppk/bb2spoiler_se') || $this->phpbb_extension_manager->is_enabled('ppk/bb2spoiler')) ? true : false;
			$is_bb3spoiler_installed=$this->phpbb_extension_manager->is_enabled('ppk/bb3spoiler') ? true : false;
			$is_coloredpostheader_installed=$this->phpbb_extension_manager->is_enabled('origin/coloredpostheader') ? true : false;
			$is_number_installed=$this->phpbb_extension_manager->is_enabled('phpbbworld/number') ? true : false;

			$is_firstpostoneverypage_installed=$this->phpbb_extension_manager->is_enabled('rxu/firstpostoneverypage') && isset($topic_data['topic_first_post_show']) && isset($topic_data['first_post_always_show']) ? true : false;
			$first_post_on_every_page=$is_firstpostoneverypage_installed && ($topic_data['topic_first_post_show'] || $topic_data['first_post_always_show']) ? 1 : 0;

			$this->template->assign_vars(array(
				'S_BB2SPOILER_INSTALLED' => $is_bb2spoiler_installed,
				'S_BB3SPOILER_INSTALLED' => $is_bb3spoiler_installed,
				'S_COLOREDPOSTHEADER_INSTALLED' => $is_coloredpostheader_installed,
				'S_NUMBER_INSTALLED' => $is_number_installed,

				'S_FIRST_POST_ON_EVERY_PAGE' => $first_post_on_every_page,
				'S_BB3AJAXER_VIEWTOPIC' => true,

				'S_BB3AJAXER_CURRENT' => $current,
				'S_BB3AJAXER_MAX' => $max,
				'S_BB3AJAXER_PER_PAGE' => $per_page,
				'S_BB3AJAXER_START' => $start,
				'S_BB3AJAXER_SORT' => $sort_dir,

				'U_BB3AJAXER_POSTS_PREV' => !$start || $user_option['prev']!=1 || $user_option['p']==-1 ? false : generate_board_url().'/viewtopic.'.$this->php_ext."?f=$forum_id&t=$topic_id&start=",
				)
			);

			if($user_option['p']==-1)
			{
				return false;
			}
			else if($user_option['p']==1)
			{
				$this->template->assign_vars(array(
					'S_USER_BB3AJAXER_POSTS' => true,
					)
				);
			}
			else
			{
				$this->template->assign_vars(array(
					'U_BB3AJAXER_POSTS' => $current==$max ? false : generate_board_url().'/viewtopic.'.$this->php_ext."?f=$forum_id&t=$topic_id&start=",
					)
				);
			}
		}
	}

	public function bb3ajaxer_ucp_tpl($event)
	{
		$this->user->add_lang_ext('ppk/bb3ajaxer', 'bb3ajaxer');

		if(isset($event['user_row']))
		{
			$user_row=$event['user_row'];
			$user_options=$this->get_bb3ajaxer_config($user_row['user_bb3ajaxer']);
		}
		else
		{
			$user_options=$this->get_bb3ajaxer_config();
		}

		$this->template->assign_vars(array(
			'S_USER_BB3AJAXER_TOPICS' => $user_options['t'],
			'S_USER_BB3AJAXER_POSTS' => $user_options['p'],
			'S_USER_BB3AJAXER_MEMBERS' => $user_options['m'],
			'S_USER_BB3AJAXER_SEARCH' => $user_options['s'],
			'S_USER_BB3AJAXER_PREVIOUS' => $user_options['prev'],

			'S_BB3AJAXER_TOPICS_APPLY' => $this->bb3ajaxer_config_apply['t'],
			'S_BB3AJAXER_POSTS_APPLY' => $this->bb3ajaxer_config_apply['p'],
			'S_BB3AJAXER_MEMBERS_APPLY' => $this->bb3ajaxer_config_apply['m'],
			'S_BB3AJAXER_SEARCH_APPLY' => $this->bb3ajaxer_config_apply['s'],
			'S_BB3AJAXER_PREVIOUS_APPLY' => $this->bb3ajaxer_config_apply['prev'],
			)
		);
	}

	public function bb3ajaxer_ucp_data($event)
	{
		$data=$event['data'];

		$user_bb3ajaxer=$this->request->variable('user_bb3ajaxer', array(''=>0));

		$bb3ajaxer_config=$this->bb3ajaxer_config;
		foreach($bb3ajaxer_config as $k => $v)
		{
			$bb3ajaxer_config[$k]=isset($user_bb3ajaxer[$k]) ? ($user_bb3ajaxer[$k] ? 1 : 0) : 0;
		}
		$data['user_bb3ajaxer']=$bb3ajaxer_config;

		$event['data']=$data;
	}

	public function bb3ajaxer_ucp_sql($event)
	{
		$sql_ary=$event['sql_ary'];
		$data=$event['data'];

		$sql_ary['user_bb3ajaxer']=json_encode($data['user_bb3ajaxer']);

		$event['sql_ary']=$sql_ary;
	}

	public function get_on_page($per_page, $start)
	{
		return floor((int) $start / (int) $per_page) + 1;
	}

	public function on_page($num_items, $per_page, $start)
	{
		return max(ceil($num_items / $per_page), 1);
	}

	public function validate_start($start, $per_page, $num_items)
	{
		if ($start < 0 || $start >= $num_items)
		{
			return ($start < 0 || $num_items <= 0) ? 0 : floor(($num_items - 1) / $per_page) * $per_page;
		}

		return $start;
	}

	public function get_bb3ajaxer_config($user_data=false)
	{
		$config=$user_data!==false ? $user_data : $this->user->data['user_bb3ajaxer'];

		$bb3ajaxer_config=$config ? json_decode($config, true) : array();

		foreach($this->bb3ajaxer_config as $k => $v)
		{
			if($k=='prev')
			{
				$bb3ajaxer_config[$k]=$this->bb3ajaxer_config_apply[$k]==1 ? $this->bb3ajaxer_config[$k] : (isset($bb3ajaxer_config[$k]) ? $bb3ajaxer_config[$k] : $this->bb3ajaxer_config[$k]);
			}
			else
			{
				if($this->bb3ajaxer_config_apply[$k]==1)
				{
					$bb3ajaxer_config[$k]=!$this->bb3ajaxer_config[$k] ? -1 : ($this->bb3ajaxer_config[$k]==-1 ? 0 : 1);
				}
				else
				{
					$bb3ajaxer_config[$k]=isset($bb3ajaxer_config[$k]) ? $bb3ajaxer_config[$k] : (!$this->bb3ajaxer_config[$k] ? -1 : ($this->bb3ajaxer_config[$k]==-1 ? 0 : 1));
				}
			}
		}

		return $bb3ajaxer_config;
	}

	public function bb3ajaxer_config($value, $key)
	{
		$string='';

		$string.='<label><input type="radio" name="bb3ajaxer_config['.$key.']" value="1"'.($this->bb3ajaxer_config[$key]==1 ? ' checked="checked"' : '').' /> '.($key!='prev' ? $this->user->lang['BB3AJAXER_BY_SCROLL'] : $this->user->lang['YES']).'</label>';
		$string.='<label><input type="radio" name="bb3ajaxer_config['.$key.']" value="0"'.($this->bb3ajaxer_config[$key]==0 ? ' checked="checked"' : '').' /> '.$this->user->lang['NO'].'</label>';

		$key!='prev' ? $string.='<label><input type="radio" name="bb3ajaxer_config['.$key.']" value="-1"'.($this->bb3ajaxer_config[$key]==-1 ? ' checked="checked"' : '').' /> '.$this->user->lang['BB3AJAXER_LINK_ONLY'].'</label>' : '';

		$string.='<br />'.$this->bb3ajaxer_config_apply($value, $key);

		return $string;
	}

	public function bb3ajaxer_config_apply($value, $key)
	{
		$string='';

		$string.='<div style="padding:5px;">&rdca; <label>'.$this->user->lang['BB3AJAXER_APPLY_OPTIONS'].'
			<select name="bb3ajaxer_config_apply['.$key.']">
				<option value="0"'.($this->bb3ajaxer_config_apply[$key]==0 ? ' selected="selected"' : '').'>'.$this->user->lang['BB3AJAXER_APPLY_DEFAULT'].'</option>
				<option value="1"'.($this->bb3ajaxer_config_apply[$key]==1 ? ' selected="selected"' : '').'>'.$this->user->lang['BB3AJAXER_APPLY_FORCED'].'</option>
			</select></label></div>';

		return $string;
	}

	public function add_config($event)
	{
		$mode=$event['mode'];
		$submit=$event['submit'];
		$display_vars=$event['display_vars'];

		if($mode=='post')
		{
			if(!isset($this->user->lang['ACP_BB3AJAXER']))
			{
				$this->user->add_lang_ext('ppk/bb3ajaxer', 'bb3ajaxer_acp');
			}

			if($submit)
			{
				$bb3ajaxer_config=$this->request->variable('bb3ajaxer_config', array(''=>0));
				$bb3ajaxer_config_apply=$this->request->variable('bb3ajaxer_config_apply', array(''=>0));

				$global_config=$this->bb3ajaxer_config;
				$global_config_apply=$this->bb3ajaxer_config_apply;
				foreach($global_config as $config => $apply)
				{
					$global_config[$config]=isset($bb3ajaxer_config[$config]) ? (int) $bb3ajaxer_config[$config] : 0;
					$global_config_apply[$config]=isset($bb3ajaxer_config_apply[$config]) ? (int) $bb3ajaxer_config_apply[$config] : $apply;
				}

				$this->config->set('bb3ajaxer_config', json_encode($global_config));
				$this->config->set('bb3ajaxer_config_apply', json_encode($global_config_apply));
			}

			$new_display_vars=array();
			foreach($display_vars['vars'] as $k => $v)
			{
				$new_display_vars[$k]=$v;
				if($k=='max_post_img_height')
				{
					$new_display_vars['legend_ajaxer']='ACP_BB3AJAXER';

					$new_display_vars['t']=array('lang' => 'BB3AJAXER_TOPICS', 'validate' => 'int', 'type' => 'custom', 'function' => array($this, 'bb3ajaxer_config'), 'explain' => true,);
					$new_display_vars['p']=array('lang' => 'BB3AJAXER_POSTS', 'validate' => 'int', 'type' =>  'custom', 'function' => array($this, 'bb3ajaxer_config'), 'explain' => true,);
					$new_display_vars['m']=array('lang' => 'BB3AJAXER_MEMBERS', 'validate' => 'int', 'type' =>  'custom', 'function' => array($this, 'bb3ajaxer_config'), 'explain' => true,);
					$new_display_vars['s']=array('lang' => 'BB3AJAXER_SEARCH', 'validate' => 'int', 'type' =>  'custom', 'function' => array($this, 'bb3ajaxer_config'), 'explain' => true,);
					$new_display_vars['prev']=array('lang' => 'BB3AJAXER_PREVIOUS', 'validate' => 'int', 'type' =>  'custom', 'function' => array($this, 'bb3ajaxer_config'), 'explain' => true,);

				}
			}
			$display_vars['vars']=$new_display_vars;
		}

		$event['display_vars']=$display_vars;
	}

}
